
# gtdb_op_txt                 = '/Users/songweizhi/Desktop/retrieve_GTDB_genomes/Sponge_MAGs_1677.ar122.summary.tsv'
# gtdb_ref_gnm_folder         = '/srv/scratch/z5039045/DB/GTDB_r202/release202/fastani'

gtdb_gnm_metadata           = '/Users/songweizhi/Desktop/retrieve_GTDB_genomes/ar122_metadata_r202.tsv'
gtdb_ref_gnm_path_txt       = '/Users/songweizhi/Desktop/retrieve_GTDB_genomes/GTDB_r202_genome_paths.tsv'
taxons_to_retrieve_txt      = '/Users/songweizhi/Desktop/retrieve_GTDB_genomes/taxons_to_retrieve.txt'
